program BogusSat;

{ Copyright 1990 Bret Mulvey.  Permission granted to copy for private,
  non-commercial use.  Permission granted to upload to bulletin board
  systems as long as this copyright notice remains intact. }

{ VGA required. }

{$N+,E+}

  uses
    Crt;

  const
    colors = 17;
    levels = 15;

  procedure PutPixel(x,y,c: integer);
  begin { proc PutPixel }
    Mem[$A000:word(320*y+x)]:=c;
  end; { proc PutPixel }

  procedure initGraphics;
    var
      i,j : integer;
      s,t,r,g,b : single;
  begin { proc initGraphics }
    Inline($B8/$13/0/$CD/$10);
    Port[$3C8]:=0;
    Port[$3C9]:=0;
    Port[$3C9]:=0;
    Port[$3C9]:=0;
    for i:=0 to colors-1 do
      begin
        s:=1-i/(colors-1);
        t:=2*pi*i/colors;
        r:=1+s*sin(t+pi/4+2*pi/3)/1.5;
        g:=1+s*sin(t+pi/4-2*pi/3)/2.0;
        b:=1+s*sin(t+pi/4+0*pi/3)/1.0;
        for j:=1 to levels do
          begin
            t:=31*j/levels*(2-Sqrt(s));
            Port[$3C9]:=Trunc(t*r);
            Port[$3C9]:=Trunc(t*g);
            Port[$3C9]:=Trunc(t*b);
          end;
      end;
  end; { proc initGraphics }

  const
    px = 320;
    py = 200;

  type
    row = array [1..py] of single;
    data = array [1..px] of ^row;
    matrix = array [1..px,1..py] of byte;
    matrixPtr = ^matrix;

  var
    h : data;
    z : ^matrix;

  procedure plot(x,y: integer);
    var i : integer;
  begin { proc plot }
    if h[x]^[y]<=0 then
      i:=colors div 6
    else
      i:=Round((h[x]^[y]+1)*(colors-1)/2);
    i:=i*levels+z^[x,y];
    PutPixel(x-1,y-1,i);
  end; { proc plot }

  const
    rf = 125;

  procedure hline(x1,x2,y: integer);
    var
      x : integer;
      q : single;
  begin { proc hline }
    if x2-x1<2 then exit;
    x:=(x1+x2+1) div 2;
    q:=(h[x1]^[y]+h[x2]^[y])/2+(2*random-1)*(x2-x1)/rf+(2*random-1)*4/rf;
    if q<-1 then q:=-1
    else if q>1 then q:=1;
    h[x]^[y]:=q;
    plot(x,y);
    hline(x1,x,y);
    hline(x,x2,y);
  end; { proc hline }

  procedure vline(y1,y2,x: integer);
    var
      y : integer;
      q : single;
  begin { proc vline }
    if y2-y1<2 then exit;
    y:=(y1+y2+1) div 2;
    q:=(h[x]^[y1]+h[x]^[y2])/2+(2*random-1)*(y2-y1)/rf+(2*random-1)*4/rf;
    if q<-1 then q:=-1
    else if q>1 then q:=1;
    h[x]^[y]:=q;
    plot(x,y);
    vline(y1,y,x);
    vline(y,y2,x);
  end; { proc vline }

  procedure subdivide(x1,y1,x2,y2: integer);
    var x,y : integer;
  begin { proc subdivide }
    if keypressed then exit;
    if (x2-x1<2) or (y2-y1<2) then exit;
    if x2-x1>y2-y1 then
      begin
        x:=(x1+x2+1) div 2;
        vline(y1,y2,x);
        subdivide(x1,y1,x,y2);
        subdivide(x,y1,x2,y2);
      end
    else
      begin
        y:=(y1+y2+1) div 2;
        hline(x1,x2,y);
        subdivide(x1,y1,x2,y);
        subdivide(x1,y,x2,y2);
      end;
  end; { proc subdivide }

  PROCEDURE say(s: String);
  BEGIN { say }
    GotoXY(9,24);
    Write(s);
  END; { say }

  procedure getHeights;
    var i,j : integer;
  begin { proc getHeights }
    say('Generating altitudes...');
    New(z);
    FillChar(z^,SizeOf(z^),Round(levels*2/pi));
    for i:=1 to px do New(h[i]);
    Randomize;
    h[1]^[1]:=2*random-1; plot(1,1);
    h[px]^[1]:=2*random-1; plot(px,1);
    h[1]^[py]:=2*random-1; plot(1,py);
    h[px]^[py]:=2*random-1; plot(px,py);
    hline(1,px,1);
    vline(1,py,px);
    hline(1,px,py);
    vline(1,py,1);
    subdivide(1,1,px,py);
    say(' Generating shadows... ');
    for i:=1 to px do
    if not keypressed then
    for j:=1 to py do
      if h[i]^[j]<0 then h[i]^[j]:=-1/16;
  end; { proc getHeights }

  procedure norm(var x,y,z: single);
    var r : single;
  begin { proc norm }
    r:=Sqrt(Sqr(x)+Sqr(y)+Sqr(z));
    x:=x/r; y:=y/r; z:=z/r;
  end; { proc norm }

  procedure doLighting;
    var
      sx,sy,sz : single;
      i,j : integer;
      nx,ny,nz : single;
      k : single;
  begin { proc doLighting }
    k:=pi*random;
    sx:=cos(k); sy:=-sin(k); sz:=1;
    norm(sx,sy,sz);
    for i:=2 to px-1 do
    if not keypressed then
    for j:=2 to py-1 do
      begin
        nx:=h[i-1]^[j-1]/2+h[i-1]^[j]+h[i-1]^[j+1]/2-h[i+1]^[j-1]/2-h[i+1]^[j]-h[i+1]^[j+1]/2;
        ny:=h[i-1]^[j-1]/2+h[i]^[j-1]+h[i+1]^[j-1]/2-h[i-1]^[j+1]/2-h[i]^[j+1]-h[i+1]^[j+1]/2;
        nz:=1/3;
        norm(nx,ny,nz);
        k:=nx*sx+ny*sy+nz*sz;
        if k<0 then k:=0;
        z^[i,j]:=Trunc(1+k*levels);
        plot(i,j);
      end;
  end; { proc doLighting }

begin
  initGraphics;
  DirectVideo:=False;
  TextAttr:=$0F;
  getHeights;
  doLighting;
  IF ReadKey=#0 THEN
    IF ReadKey=#0 THEN ;
  Inline($B8/3/0/$CD/$10);
end.